@@ -28,6 +28,8 @@ from xml.etree import ElementTree as ET
 from collections import namedtuple
 from string import Template
 
+import pdb
+
 #logging
 log_format = '%(asctime)s:%(levelname)s %(message)s'
 # have to come up with a common name, instead of using the same logger
@@ -42,7 +44,8 @@ SearchResults = namedtuple("SearchResults", ["found", "not_found"])
 #    simply prepends the key with the namespace, so it becomes
 #    {http://schemas.android.com/apk/res/android}:key_name
 #    which is what the XML library expects for the tree.get() command.
-#    Example: tree.get({namespace}:name) returns value associated with android:name
+#    Example:
+#    tree.get({namespace}:name) returns value associated with android:name
 
 # Well known manifest namespace outlined here:
 # http://developer.android.com/guide/topics/manifest/manifest-element.html
@@ -155,6 +158,11 @@ _required_provider_attributes = [
     ANDROID_CONTENT_PROVIDER_MULTIPROCESS
 ]
 
+MISSING_PROPERTY = """
+Missing {property} property in properties file. Unable to create verification
+request to use against the API. Check you airshipconfig.properties file for
+errors"""
+
 
 def get_verification_request(android_properties):
     """Get the verification request.
@@ -165,14 +173,25 @@ def get_verification_request(android_properties):
     :type key: urllib2.request
     """
 
-    isProduction = android_properties[ANDROID_CONFIG_IS_PROD_KEY]
-    if isProduction:
-        appKey = android_properties[ANDROID_CONFIG_PROD_APP_KEY]
-        appSecret = android_properties[ANDROID_CONFIG_PROD_APP_SECRET_KEY]
-    else:
-        appKey = android_properties[ANDROID_CONFIG_DEV_APP_KEY]
-        appSecret = android_properties[ANDROID_CONFIG_DEV_APP_SECRET_KEY]
-    return support.get_base_verification_request(appKey=appKey, appSecret=appSecret)
+    isProduction = android_properties.get(ANDROID_CONFIG_IS_PROD_KEY, None)
+    if isProduction is None:
+        log.info("airshipconfig.properties is missing isProduction key"
+                 "Assuming development")
+        isProduction = False
+
+    try:
+        if isProduction:
+            appKey = android_properties[ANDROID_CONFIG_PROD_APP_KEY]
+            appSecret = android_properties[ANDROID_CONFIG_PROD_APP_SECRET_KEY]
+        else:
+            appKey = android_properties[ANDROID_CONFIG_DEV_APP_KEY]
+            appSecret = android_properties[ANDROID_CONFIG_DEV_APP_SECRET_KEY]
+    except KeyError as error:
+        log.error(MISSING_PROPERTY.format(property=error))
+        return None
+
+    return support.get_base_verification_request(appKey=appKey,
+                                                 appSecret=appSecret)
 
 
 def namespace_attribute_key(key, namespace=ANDROID_XMLNS):
@@ -190,12 +209,13 @@ def namespace_attribute_key(key, namespace=ANDROID_XMLNS):
     values = dict(namespace=namespace, key=key)
     return namespace_key_template.substitute(values)
 
+def parse_manifest_in_directory(project_directory):
+    """Recursively search the given directory for the AndroidManifest
 
-def parse_android_manifest(project_directory):
-    """Parse the values of the android manifest
-
-    :param project_directory: Path to the root directory of the Android project to test
-    :return: ElementTree with the contents of the AndroidManifest
+    :param project_directory: Root directory of the project
+    :return: Returns the first manifest found. If there are multiple manifests
+        the result is one of those. Use parse_android_manifest for that
+        situation.
     :rtype: xml.etree.ElementTree
     """
 
@@ -209,8 +229,21 @@ def parse_android_manifest(project_directory):
                                                        project_directory))
         return None
 
+    return parse_android_manifest(path)
+
+
+def parse_android_manifest(manifest_path):
+    """Parse the values of the android manifest
+
+    :param manifest_path: Path to the root directory of the Android project
+        to test
+
+    :return: ElementTree with the contents of the AndroidManifest
+    :rtype: xml.etree.ElementTree
+    """
+
     try:
-        tree = ET.parse(path)
+        tree = ET.parse(manifest_path)
     except ET.ParseError as parseError:
         log.error("Error parsing AndroidManifest {}".format(parseError.message))
         return None
@@ -256,9 +289,20 @@ Please add the required permission to the manifest.
 """
 
 
-def is_missing_android_uses_permissions(manifest_element_tree):
+def is_missing_uses_permissions(manifest_element_tree):
     """Check the permissions of type uses-permission
 
+    Looking for similar xml:
+
+    .. code-block:: xml
+
+        <uses-permission android:name="android.permission.VIBRATE"/>
+        <uses-permission android:name="com.google.android.c2dm.permission.RECEIVE"/>
+        <uses-permission android:name="android.permission.ACCESS_NETWORK_STATE"/>
+        <uses-permission android:name="android.permission.WAKE_LOCK"/>
+        <uses-permission android:name="android.permission.INTERNET"/>
+        <uses-permission android:name="android.permission.GET_ACCOUNTS"/>
+
     :param manifest_element_tree: Tree for the AndroidManifest
     :type key: ElementTree
     :returns: True if there are missing permissions, false otherwise
@@ -306,7 +350,8 @@ def is_missing_package_dependent_permissions(manifest_element_tree,
         namespace_attribute_key(ANDROID_MANIFEST_NAME_KEY)
     package_based_permission_is_present = False
 
-    # Looking for the <permission android:name "package.name" android:protectionLevel:"signature"
+    # Looking for the
+    # <permission android:name "package.name" android:protectionLevel:"signature"
     for permission in root.iter("permission"):
         name = permission.get(package_based_permission)
         protection_level = permission.get(namespace_attribute_key(
@@ -323,7 +368,8 @@ def is_missing_package_dependent_permissions(manifest_element_tree,
         msg = MISSING_ATTRIBUTE.format(missing_permission=msg)
         log.error(msg)
 
-    # Looking for   <uses-permission android:name="package.foo.permission.C2D_MESSAGE" />
+    # Looking for
+    # <uses-permission android:name="package.foo.permission.C2D_MESSAGE" />
     package_based_permission_is_present = False
     for permission in root.iter("uses-permission"):
         name = permission.get(package_based_permission)
@@ -343,9 +389,17 @@ def is_missing_package_dependent_permissions(manifest_element_tree,
         return False
 
 
-def is_missing_application_receiver_attributes(manifest_element_tree):
+def is_missing_receiver_attributes(manifest_element_tree):
     """Check required application receiver attributes
 
+    Looking for similar XML
+
+    .. code-block:: xml
+
+        <receiver android:name="com.urbanairship.push.GCMPushReceiver" android:permission="com.google.android.c2dm.permission.SEND">
+        <receiver android:name="com.urbanairship.push.GCMPushReceiver" android:permission="com.google.android.c2dm.permission.SEND">
+        <receiver android:name="com.google.android.c2dm.permission.SEND" />
+
     :param manifest_element_tree: Tree for the AndroidManifest
     :type key: ElementTree
     :return: True if required permissions are present, false if not
@@ -370,13 +424,12 @@ def is_missing_application_receiver_attributes(manifest_element_tree):
         if attribute == core_receiver:
             found_receiver_attributes.append(attribute)
 
-
     # Search for <receiver android:name="com.urbanairship.push.GCMPushReceiver"
     # android:permission="com.google.android.c2dm.permission.SEND">
     # and check both values. Just add the android:name value if found to list
     # of found values
     gcm_push_receiver = ANDROID_RECEIVER_GCM_PUSH_RECEIVER
-    target_permission = "<receiver android:name=\"com.urbanairship.push.GCMPushReceiver\" android:permission=\"com.google.android.c2dm.permission.SEND\">"
+    target_permission = "<receiver android:name=\"com.urbanairship.push.GCMPushReceiver\" android:permission=\"com.google.android.c2dm.permission.SEND/\">"
     for receiver_attrib in root.iter("receiver"):
         value = receiver_attrib.get(name)
         if value == gcm_push_receiver:
@@ -397,7 +450,7 @@ def is_missing_application_receiver_attributes(manifest_element_tree):
 
     # Log results, formatted to look like XML
     core_receiver_xml = "{}<receiver android:name=\"{}\" />"
-    receiver_xml = "{}<receiver android:name=\"{}\" android:permission=\"{}\">"
+    receiver_xml = "{}<receiver android:name=\"{}\" android:permission=\"{}/\">"
 
     for result in search_results.found:
         if result == ANDROID_RECEIVER_GCM_PUSH_RECEIVER_PERMISSION:
@@ -423,9 +476,17 @@ def is_missing_application_receiver_attributes(manifest_element_tree):
         return False
 
 
-def is_missing_application_intent_filter_action_attributes(manifest_element_tree):
+def is_missing_action_attributes(manifest_element_tree):
     """Check required intent filters have the required actions
 
+    Looking for the similar xml
+
+    .. code-block:: xml
+
+        <action android:name="com.google.android.c2dm.intent.REGISTRATION"/>
+        <action android:name="android.intent.action.PACKAGE_REPLACED"/>
+        <action android:name="com.google.android.c2dm.intent.RECEIVE"/>
+
     :param manifest_element_tree: Tree for the AndroidManifest
     :type key: ElementTree
     :return: True if filters are missing, false otherwise
@@ -534,6 +595,10 @@ def required_provider_attributes(package_name):
 
     return required_values
 
+PACKAGE_NAME_MISMATCH = """
+"Package name in manifest {app} does not match {server} on UA servers"
+"""
+
 
 def is_airship_configured_properly(airship_api_response,
                                    airship_config_properties,
@@ -545,7 +610,9 @@ def is_airship_configured_properly(airship_api_response,
 
     :param airship_api_response: The json response from UA
     :type key: dict
-    :param airship_config_properties: Properties from the airshipconfig.properties file
+    :param airship_config_properties: Properties from the
+        airshipconfig.properties file
+        
     :type key: dict
     :param application_package_name: Package name for the application
     :type key: str
@@ -554,11 +621,11 @@ def is_airship_configured_properly(airship_api_response,
     """
 
     tests = []
+    server_package_name = airship_api_response[UA_API_RESPONSE_PACKAGE_KEY]
+    if server_package_name != application_package_name:
 
-    if airship_api_response[UA_API_RESPONSE_PACKAGE_KEY] != application_package_name:
-        log.error("Package name in manifest {} does not match {} on UA servers".format(
-            application_package_name,
-            airship_api_response[UA_API_RESPONSE_PACKAGE_KEY]))
+        log.info(PACKAGE_NAME_MISMATCH.format(app=application_package_name,
+                                              server=server_package_name))
         tests.append(False)
     else:
         log.info("Package name is configured on UA and matches")
@@ -602,7 +669,7 @@ def read_properties_file(root_directory):
     """
 
     path_iterator = find_file_from_dir(root_directory,
-        ANDROID_AIRSHIP_PROPERTIES_FILENAME)
+                                       ANDROID_AIRSHIP_PROPERTIES_FILENAME)
     try:
         airship_config_file = path_iterator.next()
     except StopIteration:
@@ -627,8 +694,13 @@ def read_properties_file(root_directory):
             log.debug("Adding config properties key:{} val:{}".format(key, value))
             # strip keys and values of whitespace
             properties[key] = value
-
-        match = re.match('[Yy]', properties[ANDROID_CONFIG_IS_PROD_KEY])
+        # TODO this will throw an exception if the key is not present
+        match = None
+        try:
+            match = re.match('[Yy]', properties[ANDROID_CONFIG_IS_PROD_KEY])
+        except KeyError:
+            log.info("Missing isProduction key in AirshipConfig.properties. "
+                     "Defaulting to development.")
         if match is None:
             properties[ANDROID_CONFIG_IS_PROD_KEY] = False
         else:
@@ -687,7 +759,8 @@ def is_analytics_implemented(root_path):
             airship_config[ANDROID_CONFIG_ANALYTICS_ENABLED_KEY]
     except KeyError:
         log.info(
-            "analyticsEnabled key not found in properties file, defaulting to true")
+            "analyticsEnabled key not found in properties file, "
+            "defaulting to true")
 
     if is_analytics_enabled_string is not None:
         if is_analytics_enabled_string in STRING_TRUE:
@@ -697,7 +770,9 @@ def is_analytics_implemented(root_path):
 
     if is_analytics_enabled is not True:
         log.info(
-            "Analytics has been purposefully disabled in airshipconfig.properties")
+            "Analytics has been purposefully disabled in "
+            "airshipconfig.properties")
+
         return False
 
     java_file_generator = find_file_from_dir(root_path, "*.java")
@@ -724,8 +799,9 @@ def is_analytics_implemented(root_path):
                     files_with_analytics.append(java_file_path)
 
     if len(files_with_analytics) == 0 and is_analytics_enabled is True:
-        log.error(
-            "Analytics are enabled, but no files extend an instrumented activity")
+        log.error("Analytics are enabled, but no files extend an "
+                  "instrumented activity")
+
         return False
 
     return True
