@@ -28,12 +28,21 @@ import errno
 from collections import namedtuple
 from xml.parsers.expat import ExpatError
 
+DEFAULT_CONFIG_PLIST_NAME = "AirshipConfig.plist"
+
 # AirshipConfig.plist keys
-APP_STORE_OR_AD_HOC = "APP_STORE_OR_AD_HOC_BUILD"
-DEVELOPMENT_APP_KEY = "DEVELOPMENT_APP_KEY"
-DEVELOPMENT_APP_SECRET = "DEVELOPMENT_APP_SECRET"
-PRODUCTION_APP_SECRET = "PRODUCTION_APP_SECRET"
-PRODUCTION_APP_KEY = "PRODUCTION_APP_KEY"
+APP_STORE_OR_AD_HOC_BUILD = "inProduction"
+DEVELOPMENT_APP_KEY = "developmentAppKey"
+DEVELOPMENT_APP_SECRET = "developmentAppSecret"
+PRODUCTION_APP_SECRET = "productionAppSecret"
+PRODUCTION_APP_KEY = "productionAppKey"
+
+DEPRECATED_APP_STORE_OR_AD_HOC = "APP_STORE_OR_AD_HOC_BUILD"
+DEPRECATED_DEVELOPMENT_APP_KEY = "DEVELOPMENT_APP_KEY"
+DEPRECATED_DEVELOPMENT_APP_SECRET = "DEVELOPMENT_APP_SECRET"
+DEPRECATED_PRODUCTION_APP_SECRET = "PRODUCTION_APP_SECRET"
+DEPRECATED_PRODUCTION_APP_KEY = "PRODUCTION_APP_KEY"
+
 
 # API JSON Response Keys, currently the API call returns the bundle id and
 # a boolean indicating whether or not th app is in production
@@ -133,12 +142,13 @@ to XML plist
 """
 
 AIRSHIP_CONFIG_MISSING_MSG = """
-Error opening the plist at path %s, did you forget to
+Error opening the plist at path {} with name {}, did you forget to
 add it to the bundle?
 """
 
 
-def extract_airship_config_from_app(application_path):
+def extract_airship_config_from_app(application_path,
+                                    plist_name=DEFAULT_CONFIG_PLIST_NAME):
     """ Attempts to read the AirshipConfig.plist out of th app bundle
 
     Makes the assumption the file is called AirshipConfig.plist and
@@ -146,20 +156,30 @@ def extract_airship_config_from_app(application_path):
 
     :param application_path: Path to the .app application package
     :type key: str
+    :param plist_name: Configuration plist file name, defaults to
+        "AirshipConfig.plist"
     :returns: AirshipConfig.plist as a dictionary
     :rtype: dict
     """
-    airship_config_plist =\
-        support.get_value_from_config(support.AIRSHIP_CONFIG_PLIST)
+
+    log.debug("Extracting {} at path {}".format(plist_name, application_path))
+    if plist_name is DEFAULT_CONFIG_PLIST_NAME:
+        airship_config_plist =\
+            support.get_value_from_config(support.AIRSHIP_CONFIG_PLIST)
+    else:
+        airship_config_plist = plist_name
 
     path_to_plist = os.path.join(application_path, airship_config_plist)
 
+    plist = None
     try:
         plist = plistlib.readPlist(path_to_plist)
+        log.debug("AirshipConfig.plist:%s", plist)
 
     except IOError as error:
         if error.errno == errno.ENOENT:
-            log.error(AIRSHIP_CONFIG_MISSING_MSG)
+            log.error(AIRSHIP_CONFIG_MISSING_MSG.format(application_path,
+                                                        plist_name))
             return None
 
     except ExpatError:
@@ -170,10 +190,48 @@ def extract_airship_config_from_app(application_path):
         else:
             log.info("Plist conversion successful.")
 
-    log.debug("AirshipConfig.plist:%s", plist)
+    # Check for key change in LIB-608/DEVEXP-55, changes keys from CAMEL_CAPS
+    # to lowerPascalCase for KVC property setting on iOS
+    if plist.get("productionAppSecret") is None:
+        plist = update_plist_keys_to_kvc_from_upper_snake(plist)
+
     return plist
 
 
+def update_plist_keys_to_kvc_from_upper_snake(plist):
+    """Take a Plist with deprecated keys and update it
+
+    Changing from:
+    APP_STORE_OR_AD_HOC = "APP_STORE_OR_AD_HOC_BUILD"
+    DEVELOPMENT_APP_KEY = "DEVELOPMENT_APP_KEY"
+    DEVELOPMENT_APP_SECRET = "DEVELOPMENT_APP_SECRET"
+    PRODUCTION_APP_SECRET = "PRODUCTION_APP_SECRET"
+    PRODUCTION_APP_KEY = "PRODUCTION_APP_KEY"
+
+    To:
+    APP_STORE_OR_AD_HOC_BUILD = "inProduction"
+    DEVELOPMENT_APP_KEY = "developmentAppKey"
+    DEVELOPMENT_APP_SECRET = "developmentAppSecret"
+    PRODUCTION_APP_SECRET = "productionAppSecret"
+    PRODUCTION_APP_KEY = "productionAppKey"
+
+    :param plist: Plist with keys in the form CAPS_WITH_UNDERSCORE
+    :return: Plist with KVC compliant keys, lowerCamelCase"""
+
+    updated_plist = dict()
+    updated_plist[APP_STORE_OR_AD_HOC_BUILD] = \
+        plist[DEPRECATED_APP_STORE_OR_AD_HOC]
+    updated_plist[DEVELOPMENT_APP_KEY] = \
+        plist[DEPRECATED_DEVELOPMENT_APP_KEY]
+    updated_plist[DEVELOPMENT_APP_SECRET] = \
+        plist[DEPRECATED_DEVELOPMENT_APP_SECRET]
+    updated_plist[PRODUCTION_APP_SECRET] = \
+        plist[DEPRECATED_PRODUCTION_APP_SECRET]
+    updated_plist[PRODUCTION_APP_KEY] = \
+        plist[DEPRECATED_PRODUCTION_APP_KEY]
+    return updated_plist
+
+
 def convert_deprecated_plist_at_path(path_to_plist):
     """Extract a deprecated plist from an app adn convert it
 
@@ -390,7 +448,7 @@ class VerificationInformation(object):
         :rtype: bool
         """
 
-        is_production = self.airship_config_plist[APP_STORE_OR_AD_HOC]
+        is_production = self.airship_config_plist[APP_STORE_OR_AD_HOC_BUILD]
         # TODO
         # If is_production is not a bool, then this is a non xml plist, and
         # we look for a string YES/NO. Here we check for a string, and
