@@ -26,6 +26,15 @@ import androidsupport
 
 from iossupport import VerificationInformation
 
+
+def display_version():
+    """Display the library version to stdout"""
+
+    import pkg_resources
+    version = pkg_resources.require("uaverify")[0].version
+    print("Version %s" % version)
+    return support.EXIT_SUCCESS
+
 NO_CONFIG_PLIST_ERROR = """
 No AirshipConfig.plist in app bundle. Without the configuration plist the
 verification API call cannot be made. Parsing the app key and secret from
@@ -36,10 +45,7 @@ code is slated for a future release.
 class IosVerify(object):
     """ Verify the build with Urban Airship """
 
-    def __init__(self, application_path, diagnostic_build=False):
-        self.verification_info = None
-        self.application_path = application_path
-        self.diagnostic_build = diagnostic_build
+    def __init__(self):
         iossupport.clean()
 
     def execute_xcode_verify(self):
@@ -47,12 +53,19 @@ class IosVerify(object):
 
         # Setup logger
         log = logging.getLogger(support.UA_LOGGER)
-        if self.diagnostic_build:
+
+        if self.diagnostic:
             log.info("Setting up diagnostic logging")
             support.setup_diagnostic_logging()
 
         # Extract entitlement info from app
         log.info("Executing codesign system call")
+
+        if self.application_path is None:
+            log.error("Path to application is required. application_path must"
+                      " be set in command line options")
+            return False
+
         success = iossupport.execute_codesign_system_call(
             self.application_path)
         if not success:
@@ -70,8 +83,13 @@ class IosVerify(object):
             return False
 
         # Read the AirshipConfig from the app
-        airship_config_plist = iossupport.extract_airship_config_from_app(
-            self.application_path)
+        if self.plist is None:
+            airship_config_plist = iossupport.extract_airship_config_from_app(
+                self.application_path)
+        else:
+            airship_config_plist = \
+                iossupport.extract_airship_config_from_app(
+                    self.application_path, self.plist)
 
         # Store entitlement and config info
         self.verification_info = VerificationInformation(
@@ -87,7 +105,7 @@ class IosVerify(object):
         # before the matching API call. Extracting the information from code
         # is slated for a future release.
         if not airship_config_plist:
-            if self.diagnostic_build:
+            if self.diagnostic:
                 self.verification_info.log_diagnostic_information()
             log.error(NO_CONFIG_PLIST_ERROR)
             return False
@@ -111,7 +129,7 @@ class IosVerify(object):
             log.error("Bundle id mismatch")
             return False
 
-        if self.diagnostic_build:
+        if self.diagnostic:
             # Print diagnostic info, this has the side effect of printing
             # to a file
             self.verification_info.log_diagnostic_information()
@@ -122,17 +140,11 @@ class IosVerify(object):
 class AndroidVerify(object):
     """Verify an Android Project"""
 
-    def __init__(self):
-        # These are populated with the arg parser parse_args(namespace=) method
-        # after init
-        self.application_path = None
-        self.diagnostic = False
-        self.manifest_path = None
-
     def execute_android_verify(self):
         """Verify the android build"""
 
         log = support.log
+
         if self.diagnostic:
             log.info("Setting up diagnostic logging")
             support.setup_diagnostic_logging()
@@ -244,49 +256,65 @@ def ios_main():
         argparse.ArgumentParser(description="Verify a build with Urban Airship",
                                 epilog=IOS_EPILOG)
 
-    parser.add_argument('application_path', type=str,
+    parser.add_argument('application_path', type=str, nargs="?",
                         help=APPLICATION_PATH_MESSAGE)
 
     parser.add_argument('-d', '--diagnostic', action="store_true",
                         help="Write out diagnostic information to a file")
 
+    parser.add_argument('--plist', type=str,
+                help="Name of the configuration plist, "
+                     "defaults to \"AirshipConfig.plist\" ")
+
+    parser.add_argument('-v', '--version', action="store_true",
+                        help="Display version number and exit")
+
     args = parser.parse_args()
-    verify = IosVerify(application_path=args.application_path,
-                       diagnostic_build=args.diagnostic)
+
+    # Short circuit if there are no command line args
+    if len(sys.argv) <= 1:
+        display_version()
+        parser.print_usage()
+        return support.EXIT_SUCCESS
 
     # This appends the args to the object as instance variables
+    verify = IosVerify()
     parser.parse_args(namespace=verify)
+
+    if verify.version:
+        display_version()
+        return support.EXIT_SUCCESS
+
     success = verify.execute_xcode_verify()
     # There is the possibility of a file logger
     logging.shutdown()
     if success:
-        log.info("Successful verification")
+        log.debug("Successful verification")
         return support.EXIT_SUCCESS
     else:
         log.error("Failed verification")
         return support.EXIT_FAILURE
 
 ANDROID_EPILOG = """
-Extract the entitlements from a .app file, parse them, and
-check for errors. An API call is made to Urban Airship using the
-configured key/secret pair to return API app settings for
-comparison.
+Parse the AndroidManifest.xml and look for required settings. Search
+code files for proper implementation of analytics and references to takeOff. An
+API call is made to Urban Airship using the key/secret pair to return API
+app settings for comparison.
 """
+
 ANDROID_MANIFEST_HELP = """
 If your application has more than one AndroidManifest.xml file, use this flag
 to pass the path of the manifest file to use for verification.
 """
 
-def android_main():
-    """Parses arguments and runs the uav-android tool against an app"""
-
-    log = logging.getLogger(support.UA_LOGGER)
+def android_arg_parser():
+    """Setup the arg parser for the Android tool"""
 
     parser = \
         argparse.ArgumentParser(description="Verify a build with Urban Airship",
                                 epilog=ANDROID_EPILOG)
 
-    parser.add_argument('application_path', type=str,
+    parser.add_argument('application_path', type=str, nargs="?",
                         help=APPLICATION_PATH_MESSAGE)
 
     parser.add_argument('-d', '--diagnostic', action="store_true",
@@ -295,16 +323,37 @@ def android_main():
     parser.add_argument('-m', '--manifest_path', action="store",
                         help=ANDROID_MANIFEST_HELP)
 
-    args = parser.parse_args()
+    parser.add_argument('-v', '--version', action="store_true",
+                        help="Display version number and exit")
+    return parser
+
+
+def android_main():
+    """Parses arguments and runs the uav-android tool against an app"""
+
+    log = logging.getLogger(support.UA_LOGGER)
+
+    parser = android_arg_parser()
+
+    # Short circuit if there are no command line args
+    if len(sys.argv) <= 1:
+        display_version()
+        parser.print_usage()
+        return support.EXIT_SUCCESS
 
     verify = AndroidVerify()
     parser.parse_args(namespace=verify)
+
+    if verify.version:
+        display_version()
+        return support.EXIT_SUCCESS
+
     success = verify.execute_android_verify()
     # There is the possibility of a file logger
     logging.shutdown()
-    if success == 0:
-        log.info("Successful verification")
+    if success == support.EXIT_SUCCESS:
+        log.debug("Successful verification")
         return support.EXIT_SUCCESS
     else:
-        log.info("Verification failed")
+        log.error("Verification failed")
         return support.EXIT_FAILURE
